@@ -11,9 +11,12 @@ def assert_cassette_has_one_response(cass):
     assert cass.play_count == 1
 
 
-def assert_is_json(a_string):
+def assert_is_json_bytes(b: bytes):
+    assert isinstance(b, bytes)
+
     try:
-        json.loads(a_string.decode('utf-8'))
-    except Exception:
-        assert False
+        json.loads(b)
+    except Exception as error:
+        raise AssertionError() from error
+
     assert True
