@@ -1,20 +1,22 @@
-from unittest import mock
+import http.client as httplib
 import os
+from pathlib import Path
+from unittest import mock
 
 import pytest
-import http.client as httplib
 
 from vcr import VCR, mode, use_cassette
+from vcr.patch import _HTTPConnection, force_reset
 from vcr.request import Request
 from vcr.stubs import VCRHTTPSConnection
-from vcr.patch import _HTTPConnection, force_reset
 
 
 def test_vcr_use_cassette():
     record_mode = mock.Mock()
     test_vcr = VCR(record_mode=record_mode)
     with mock.patch(
-        "vcr.cassette.Cassette.load", return_value=mock.MagicMock(inject=False)
+        "vcr.cassette.Cassette.load",
+        return_value=mock.MagicMock(inject=False),
     ) as mock_cassette_load:
 
         @test_vcr.use_cassette("test")
@@ -40,7 +42,7 @@ def test_vcr_use_cassette():
 
 
 def test_vcr_before_record_request_params():
-    base_path = "http://httpbin.org/"
+    base_path = "http://whatever.test/"
 
     def before_record_cb(request):
         if request.path != "/get":
@@ -70,16 +72,19 @@ def test_vcr_before_record_request_params():
 
         # Test filter_headers
         request = Request(
-            "GET", base_path + "?foo=bar", "", {"cookie": "test", "other": "fun", "bert": "nobody"}
+            "GET",
+            base_path + "?foo=bar",
+            "",
+            {"cookie": "test", "other": "fun", "bert": "nobody"},
         )
         assert cassette.filter_request(request).headers == {"other": "fun", "bert": "ernie"}
 
         # Test ignore_hosts
-        request = Request("GET", "http://www.test.com" + "?foo=bar", "", {"cookie": "test", "other": "fun"})
+        request = Request("GET", "http://www.test.com?foo=bar", "", {"cookie": "test", "other": "fun"})
         assert cassette.filter_request(request) is None
 
         # Test ignore_localhost
-        request = Request("GET", "http://localhost:8000" + "?foo=bar", "", {"cookie": "test", "other": "fun"})
+        request = Request("GET", "http://localhost:8000?foo=bar", "", {"cookie": "test", "other": "fun"})
         assert cassette.filter_request(request) is None
 
     with test_vcr.use_cassette("test", before_record_request=None) as cassette:
@@ -95,7 +100,6 @@ def test_vcr_before_record_response_iterable():
 
     # Prevent actually saving the cassette
     with mock.patch("vcr.cassette.FilesystemPersister.save_cassette"):
-
         # Baseline: non-iterable before_record_response should work
         mock_filter = mock.Mock()
         vcr = VCR(before_record_response=mock_filter)
@@ -119,7 +123,6 @@ def test_before_record_response_as_filter():
 
     # Prevent actually saving the cassette
     with mock.patch("vcr.cassette.FilesystemPersister.save_cassette"):
-
         filter_all = mock.Mock(return_value=None)
         vcr = VCR(before_record_response=filter_all)
         with vcr.use_cassette("test") as cassette:
@@ -133,7 +136,6 @@ def test_vcr_path_transformer():
 
     # Prevent actually saving the cassette
     with mock.patch("vcr.cassette.FilesystemPersister.save_cassette"):
-
         # Baseline: path should be unchanged
         vcr = VCR()
         with vcr.use_cassette("test") as cassette:
@@ -261,7 +263,9 @@ def test_cassette_library_dir_with_decoration_and_super_explicit_path():
 def test_cassette_library_dir_with_path_transformer():
     library_dir = "/library_dir"
     vcr = VCR(
-        inject_cassette=True, cassette_library_dir=library_dir, path_transformer=lambda path: path + ".json"
+        inject_cassette=True,
+        cassette_library_dir=library_dir,
+        path_transformer=lambda path: path + ".json",
     )
 
     @vcr.use_cassette()
@@ -360,3 +364,11 @@ def test_dynamically_added(self):
 
 TestVCRClass.test_dynamically_added = test_dynamically_added
 del test_dynamically_added
+
+
+def test_path_class_as_cassette():
+    path = Path(__file__).parent.parent.joinpath(
+        "integration/cassettes/test_httpx_test_test_behind_proxy.yml",
+    )
+    with use_cassette(path):
+        pass
